=== modified file 'NEWS'
--- NEWS	2006-05-12 06:59:47 +0000
+++ NEWS	2006-05-16 05:16:22 +0000
@@ -14,6 +14,9 @@
     progress made. 'make check' shows tests in -v mode, to be more useful
     for the PQM status window. (Robert Collins).
 
+   * 'bzr selftest --benchmark' will run a new benchmarking selftest.
+     (Robert Collins, Martin Pool).
+
 bzr 0.8  2006-05-08
 
   NOTES WHEN UPGRADING:

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-05-12 06:59:47 +0000
+++ bzrlib/builtins.py	2006-05-16 05:16:22 +0000
@@ -1779,16 +1779,20 @@
                             help='Use a different transport by default '
                                  'throughout the test suite.',
                             type=get_transport_type),
+                     Option('benchmark', help='run the bzr bencharks.'),
                     ]
 
     def run(self, testspecs_list=None, verbose=False, one=False,
-            keep_output=False, transport=None):
+            keep_output=False, transport=None, benchmark=None):
         import bzrlib.ui
-        from bzrlib.tests import selftest
+        from bzrlib.tests import selftest, benchmark_suite
         # we don't want progress meters from the tests to go to the
         # real output; and we don't want log messages cluttering up
         # the real logs.
         save_ui = bzrlib.ui.ui_factory
+        print '%10s: %s' % ('bzr', bzrlib.osutils.realpath(sys.argv[0]))
+        print '%10s: %s' % ('bzrlib', bzrlib.__path__[0])
+        print
         bzrlib.trace.info('running tests...')
         try:
             bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
@@ -1796,11 +1800,16 @@
                 pattern = '|'.join(testspecs_list)
             else:
                 pattern = ".*"
+            if benchmark:
+                test_suite_factory = benchmark_suite
+            else:
+                test_suite_factory = None
             result = selftest(verbose=verbose, 
                               pattern=pattern,
                               stop_on_failure=one, 
                               keep_output=keep_output,
-                              transport=transport)
+                              transport=transport,
+                              test_suite_factory=test_suite_factory)
             if result:
                 bzrlib.trace.info('tests passed')
             else:

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-05-09 05:36:32 +0000
+++ bzrlib/tests/__init__.py	2006-05-16 05:16:22 +0000
@@ -348,7 +348,6 @@
     accidentally overlooked.
     """
 
-    BZRPATH = 'bzr'
     _log_file_name = None
     _log_contents = ''
 
@@ -1018,15 +1017,19 @@
 
 def selftest(verbose=False, pattern=".*", stop_on_failure=True,
              keep_output=False,
-             transport=None):
+             transport=None,
+             test_suite_factory=None):
     """Run the whole test suite under the enhanced runner"""
     global default_transport
     if transport is None:
         transport = default_transport
     old_transport = default_transport
     default_transport = transport
-    suite = test_suite()
     try:
+        if test_suite_factory is None:
+            suite = test_suite()
+        else:
+            suite = test_suite_factory()
         return run_suite(suite, 'testbzr', verbose=verbose, pattern=pattern,
                      stop_on_failure=stop_on_failure, keep_output=keep_output,
                      transport=transport)
@@ -1034,9 +1037,17 @@
         default_transport = old_transport
 
 
+def benchmark_suite():
+    """Build and return a TestSuite which contains benchmark tests only."""
+    return TestSuite()
+
 
 def test_suite():
-    """Build and return TestSuite for the whole program."""
+    """Build and return TestSuite for the whole of bzrlib.
+    
+    This function can be replaced if you need to change the default test
+    suite on a global basis, but it is not encouraged.
+    """
     from doctest import DocTestSuite
 
     global MODULES_TO_DOCTEST
@@ -1114,11 +1125,6 @@
     test_transport_implementations = [
         'bzrlib.tests.test_transport_implementations']
 
-    TestCase.BZRPATH = osutils.pathjoin(
-            osutils.realpath(osutils.dirname(bzrlib.__path__[0])), 'bzr')
-    print '%10s: %s' % ('bzr', osutils.realpath(sys.argv[0]))
-    print '%10s: %s' % ('bzrlib', bzrlib.__path__[0])
-    print
     suite = TestSuite()
     # python2.4's TestLoader.loadTestsFromNames gives very poor 
     # errors if it fails to load a named module - no indication of what's

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2006-05-09 01:57:17 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2006-05-16 05:16:22 +0000
@@ -77,6 +77,16 @@
             TestOptions.current_test = None
             TestCaseInTempDir.TEST_ROOT = old_root
 
+    def test_benchmark_runs_no_tests(self):
+        """bzr selftest --benchmark should not run the default test suite."""
+        # We test this via checking that no tests are run because the
+        # benchmark suite is currently empty.
+        out, err = self.run_bzr('selftest', '--benchmark')
+        self.assertContainsRe(out, 'Ran 0 tests.*\n\nOK')
+        self.assertEqual(
+            'running tests...\nRunning tests: .\nCleaning up: .\ntests passed\n',
+            err)
+        
 
 class TestRunBzr(ExternalBase):
 

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-05-09 10:33:08 +0000
+++ bzrlib/tests/test_selftest.py	2006-05-16 05:16:22 +0000
@@ -16,6 +16,7 @@
 """Tests for the test framework."""
 
 import os
+from StringIO import StringIO
 import sys
 import unittest
 import warnings
@@ -29,6 +30,7 @@
                           TestCaseInTempDir,
                           TestCaseWithTransport,
                           TestSkipped,
+                          TestSuite,
                           TextTestRunner,
                           )
 import bzrlib.errors as errors
@@ -551,3 +553,18 @@
                               bzrlib.bzrdir.BzrDirMetaFormat1)
         self.assertIsInstance(bzrlib.bzrdir.BzrDir.open('b')._format,
                               bzrlib.bzrdir.BzrDirFormat6)
+
+
+class TestSelftest(TestCase):
+    """Tests of bzrlib.tests.selftest."""
+
+    def test_selftest_benchmark_parameter_invokes_test_suite__benchmark__(self):
+        factory_called = []
+        def factory():
+            factory_called.append(True)
+            return TestSuite()
+        out = StringIO()
+        err = StringIO()
+        self.apply_redirected(out, err, None, bzrlib.tests.selftest, 
+            test_suite_factory=factory)
+        self.assertEqual([True], factory_called)

